@@ -24,15 +24,47 @@
 #include <sys/statvfs.h>
 #include <QDebug>
 #include <QFileInfo>
+#include <QCryptographicHash>
+#include <QDirIterator>
+#include <QMessageBox>
 #include "base.h"
 
 Base::Base(MProcess &mproc, PartMan &pman, Ui::MeInstall &ui,
     const QSettings &appConf, const QCommandLineParser &appArgs)
     : QObject(ui.boxMain), proc(mproc), gui(ui), partman(pman)
 {
+    mediacheck = appArgs.isSet("media-check");
+    if (!mediacheck) nomediacheck = appArgs.isSet("no-media-check");
     populateMediaMounts = appConf.value("POPULATE_MEDIA_MOUNTPOINTS").toBool();
     nocopy = appArgs.isSet("nocopy");
     sync = appArgs.isSet("sync");
+}
+
+void Base::scanMedia()
+{
+    QSettings liveInfo("/live/config/initrd.out", QSettings::IniFormat);
+    const QString &toramMP = liveInfo.value("TORAM_MP", "/live/to-ram").toString();
+
+    // Check the installation media for errors (skip if not required).
+    bool checkmd5 = false;
+    if (!mediacheck) {
+        QFile file("/live/config/proc-cmdline");
+        if (file.open(QFile::ReadOnly | QFile::Text)) {
+            while (!file.atEnd()) {
+                if(file.readLine().trimmed() == "checkmd5") {
+                    checkmd5 = true;
+                    break;
+                }
+            }
+        }
+    }
+    if(checkmd5) proc.log("No media check (checkmd5)", MProcess::Standard);
+    else if(nomediacheck) proc.log("No media check", MProcess::Standard);
+    else {
+        QString msrc = liveInfo.value("SQFILE_DIR", "/live/boot-dev/antiX").toString();
+        if (!QFile::exists(msrc)) msrc = toramMP + "/antiX";
+        checkMediaMD5(msrc);
+    }
 
     bootSource = "/live/aufs/boot";
     rootSources << "/live/aufs/bin" << "/live/aufs/dev"
@@ -41,24 +73,23 @@ Base::Base(MProcess &mproc, PartMan &pman, Ui::MeInstall &ui,
         << "/live/aufs/sbin" << "/live/aufs/usr" << "/live/aufs/var" << "/live/aufs/home";
 
     // calculate required disk space
-    proc.exec("du", {"-sb", bootSource}, nullptr, true);
-    partman.bootSpaceNeeded = proc.readOut().section('\t', 0, 0).toLongLong();
+    proc.exec("du", {"-scb", bootSource}, nullptr, true);
+    partman.bootSpaceNeeded = proc.readOut(true).section('\n', -1).section('\t', 0, 0).toLongLong();
 
-    QSettings liveInfo("/live/config/initrd.out", QSettings::IniFormat);
-    QString linuxfs = liveInfo.value("SQFILE_FULL", "/live/boot-dev/antiX/linuxfs").toString();
-    if (!QFile::exists(linuxfs + ".info")) {
-        linuxfs = liveInfo.value("TORAM_MP", "/live/to-ram").toString() + "/antiX/linuxfs";
-    }
-    const QSettings squashInfo(linuxfs + ".info", QSettings::IniFormat);
+    QString infile = liveInfo.value("SQFILE_FULL", "/live/boot-dev/antiX/linuxfs").toString() + ".info";
+    if (!QFile::exists(infile)) infile = toramMP + "/antiX/linuxfs.info";
     bool floatOK = false;
-    partman.rootSpaceNeeded = 1024 * squashInfo.value("UncompressedSizeKB").toFloat(&floatOK);
+    if (QFile::exists(infile)) {
+        const QSettings squashInfo(infile, QSettings::IniFormat);
+        partman.rootSpaceNeeded = 1024 * squashInfo.value("UncompressedSizeKB").toFloat(&floatOK);
+    }
     if (!floatOK) {
-        rootSources.prepend("-sb");
+        rootSources.prepend("-scb");
         proc.exec("du", rootSources, nullptr, true);
         rootSources.removeFirst();
-        partman.rootSpaceNeeded = proc.readOut().section('\t', 0, 0).toLongLong();
+        partman.rootSpaceNeeded = proc.readOut(true).section('\n', -1).section('\t', 0, 0).toLongLong();
     }
-    qDebug() << "Image:" << linuxfs << partman.rootSpaceNeeded << floatOK;
+    qDebug() << "Image size:" << partman.rootSpaceNeeded << floatOK << infile;
     // Account for persistent root.
     if (QFileInfo::exists("/live/perist-root")) {
         proc.shell("df /live/persist-root --output=used --total |tail -n1", nullptr, true);
@@ -71,10 +102,85 @@ Base::Base(MProcess &mproc, PartMan &pman, Ui::MeInstall &ui,
     qDebug() << "Minimum space:" << partman.bootSpaceNeeded << "(boot)," << partman.rootSpaceNeeded << "(root)";
 }
 
-bool Base::install()
+void Base::checkMediaMD5(const QString &path)
+{
+    checking = true;
+    const QString osplash = gui.labelSplash->text();
+    const QString &nsplash = tr("Checking installation media.")
+        + "<br/><font size=2>%1% - " + tr("Press ESC to skip.") + "</font>";
+    gui.labelSplash->setText(nsplash.arg(0));
+    qint64 btotal = 0;
+    struct FileHash {
+        QString path;
+        QByteArray hash;
+    };
+    QList<FileHash> hashes;
+    static const char *failmsg = QT_TR_NOOP("The installation media is corrupt.");
+    // Obtain a list of MD5 hashes and their files.
+    QDirIterator it(path, {"*.md5"}, QDir::Files);
+    QStringList missing("linuxfs");
+    if (!QFile::exists("/live/config/did-toram") || QFile::exists("/live/config/toram-all")) {
+        missing << "vmlinuz" << "initrd.gz";
+    }
+    while (it.hasNext()) {
+        QFile file(it.next());
+        if (!file.open(QFile::ReadOnly | QFile::Text)) throw failmsg;
+        while (!file.atEnd()) {
+            QString line(file.readLine().trimmed());
+            const QString &fname = line.section(' ', 1, 1, QString::SectionSkipEmpty);
+            missing.removeOne(fname);
+            FileHash sfhash = {
+                .path = it.path() + '/' + fname,
+                .hash = QByteArray::fromHex(line.section(' ', 0, 0, QString::SectionSkipEmpty).toUtf8())
+            };
+            hashes.append(sfhash);
+            btotal += QFileInfo(sfhash.path).size();
+            qApp->processEvents();
+        }
+    }
+    if(!missing.isEmpty()) throw failmsg;
+    // Check the MD5 hash of each file.
+    const size_t bufsize = 65536;
+    std::unique_ptr<char[]> buf(new char[bufsize]);
+    qint64 bprog = 0;
+    for(const FileHash &fh : hashes) {
+        QListWidgetItem *logEntry = proc.log("Check MD5: " + fh.path, MProcess::Standard);
+        QFile file(fh.path);
+        if (!file.open(QFile::ReadOnly)) throw failmsg;
+        QCryptographicHash hash(QCryptographicHash::Md5);
+        while (checking && !file.atEnd()) {
+            const qint64 rlen = file.read(buf.get(), bufsize);
+            if(rlen < 0) throw failmsg;
+            hash.addData(buf.get(), rlen);
+            bprog += rlen;
+            gui.labelSplash->setText(nsplash.arg((bprog * 100) / btotal));
+            qApp->processEvents();
+        }
+        if (!checking) break;
+        if (hash.result() != fh.hash) throw failmsg;
+        proc.log(logEntry);
+    }
+    gui.labelSplash->setText(osplash);
+    if (!checking) proc.log("Check halted", MProcess::Standard);
+    checking = false;
+}
+
+void Base::haltCheck(bool silent)
+{
+    if (!checking) return;
+    if (!silent) {
+        QMessageBox::StandardButton rc = QMessageBox::warning(gui.boxMain, QString(),
+            tr("Are you sure you want to skip checking the installation media?"),
+            QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
+        if(rc != QMessageBox::Yes) return;
+    }
+    checking = false;
+}
+
+void Base::install()
 {
     proc.log(__PRETTY_FUNCTION__);
-    if (proc.halted()) return false;
+    if (proc.halted()) return;
     proc.advance(1, 2);
 
     if (!partman.willFormat("/")) {
@@ -84,8 +190,7 @@ bool Base::install()
         proc.shell("find /mnt/antiX -mindepth 1 -maxdepth 1 ! -name home -exec rm -r {} \\;");
 
         if (proc.exitStatus() != QProcess::NormalExit) {
-            failure = tr("Failed to delete old system on destination.");
-            return false;
+            throw QT_TR_NOOP("Failed to delete old system on destination.");
         }
     }
 
@@ -98,7 +203,7 @@ bool Base::install()
     mkdir("/mnt/antiX/sys", 0755);
     mkdir("/mnt/antiX/run", 0755);
 
-    if (!copyLinux()) return false;
+    copyLinux();
 
     proc.advance(1, 1);
     proc.status(tr("Fixing configuration"));
@@ -113,8 +218,7 @@ bool Base::install()
     // if POPULATE_MEDIA_MOUNTPOINTS is true in gazelle-installer-data, then use the --mntpnt switch
     partman.makeFstab(populateMediaMounts);
     if (!partman.fixCryptoSetup()) {
-        failure = tr("Failed to finalize encryption setup.");
-        return false;
+        throw QT_TR_NOOP("Failed to finalize encryption setup.");
     }
     // Disable hibernation inside initramfs.
     if (partman.isEncrypt("SWAP")) {
@@ -158,14 +262,12 @@ bool Base::install()
         proc.shell("/bin/mv -f /mnt/antiX/etc/rc2.d/S*virtualbox-guest-utils /mnt/antiX/etc/rc2.d/K01virtualbox-guest-utils >/dev/null 2>&1");
         proc.shell("/bin/mv -f /mnt/antiX/etc/rcS.d/S*virtualbox-guest-x11 /mnt/antiX/etc/rcS.d/K21virtualbox-guest-x11 >/dev/null 2>&1");
     }
-
-    return true;
 }
 
-bool Base::copyLinux()
+void Base::copyLinux()
 {
     proc.log(__PRETTY_FUNCTION__);
-    if (proc.halted()) return false;
+    if (proc.halted()) return;
 
     // copy most except usr, mnt and home
     // must copy boot even if saving, the new files are required
@@ -187,8 +289,7 @@ bool Base::copyLinux()
     proc.advance(80, sourceInodes);
     proc.status(tr("Copying new system"));
     // Placed here so the progress bar moves to the right position before the next step.
-    if (proc.halted()) return false;
-    if (nocopy) return true; // Skip copying on purpose
+    if (nocopy) return; // Skip copying on purpose
 
     const QString &joined = MProcess::joinCommand(prog, args);
     qDebug().noquote() << "Exec COPY:" << joined;
@@ -217,10 +318,7 @@ bool Base::copyLinux()
     qDebug() << "Exit COPY:" << proc.exitCode() << proc.exitStatus();
     if (proc.exitStatus() != QProcess::NormalExit) {
         proc.log(logEntry, -1);
-        failure = tr("Failed to copy the new system.");
-        return false;
+        throw QT_TR_NOOP("Failed to copy the new system.");
     }
     proc.log(logEntry, proc.exitCode() ? 0 : 1);
-
-    return !proc.halted(); // Reduces domino effect if the copy is aborted.
 }
