@@ -27,8 +27,8 @@
 #include <QTimeZone>
 #include "oobe.h"
 
-Oobe::Oobe(MProcess &mproc, Ui::MeInstall &ui, QWidget *parent, const QSettings &appConf)
-    : QObject(parent), proc(mproc), gui(ui), master(parent)
+Oobe::Oobe(MProcess &mproc, Ui::MeInstall &ui, QWidget *parent, const QSettings &appConf, bool oem, bool modeOOBE)
+    : QObject(parent), proc(mproc), gui(ui), master(parent), oem(oem), online(modeOOBE)
 {
     gui.textComputerName->setText(appConf.value("DEFAULT_HOSTNAME").toString());
     enableServices = appConf.value("ENABLE_SERVICES").toStringList();
@@ -51,7 +51,7 @@ Oobe::Oobe(MProcess &mproc, Ui::MeInstall &ui, QWidget *parent, const QSettings
             "-mindepth", "2", "-type", "f", "-printf", "%P\\n"}, nullptr, true);
     timeZones = proc.readOutLines();
     gui.comboTimeArea->clear();
-    for (const QString &zone : timeZones) {
+    for (const QString &zone : qAsConst(timeZones)) {
         const QString &area = zone.section('/', 0, 0);
         if (gui.comboTimeArea->findData(QVariant(area)) < 0) {
             QString text(area);
@@ -196,29 +196,34 @@ void Oobe::enable()
 
 void Oobe::process()
 {
-    if (!online) proc.setChRoot("/mnt/antiX");
-    QTreeWidgetItemIterator it(gui.treeServices);
-    for (; *it; ++it) {
-        if ((*it)->parent()) setService((*it)->text(0), (*it)->checkState(0) == Qt::Checked);
-    }
-    if (haveSamba) {
-        const bool enable = gui.checkSamba->isChecked();
-        setService("smbd", enable);
-        setService("nmbd", enable);
-        setService("samba-ad-dc", enable);
-    }
-    proc.setChRoot();
+    if (!oem) {
+        if (!online) proc.setChRoot("/mnt/antiX");
 
-    setComputerName();
-    setLocale();
-    if (haveSnapshotUserAccounts) { // skip user account creation
+        QTreeWidgetItemIterator it(gui.treeServices);
+        for (; *it; ++it) {
+            if ((*it)->parent()) setService((*it)->text(0), (*it)->checkState(0) == Qt::Checked);
+        }
+        if (haveSamba) {
+            const bool enable = gui.checkSamba->isChecked();
+            setService("smbd", enable);
+            setService("nmbd", enable);
+            setService("samba-ad-dc", enable);
+        }
+        proc.setChRoot();
+        setComputerName();
+        setLocale();
+    }
+    if (haveSnapshotUserAccounts || oem) { // skip user account creation
         proc.exec("rsync", {"-a", "/home/", "/mnt/antiX/home/",
             "--exclude", ".cache", "--exclude", ".gvfs", "--exclude", ".dbus", "--exclude", ".Xauthority",
             "--exclude", ".ICEauthority", "--exclude", ".config/session"});
     } else {
         setUserInfo();
     }
-    if (online) proc.exec("update-rc.d", {"oobe", "disable"});
+    if (online) {
+        proc.shell("sed -i 's/nosplash\\b/splash/g' /boot/grub/grub.cfg");
+        proc.exec("update-rc.d", {"oobe", "disable"});
+    }
 }
 
 /* Services */
@@ -232,7 +237,7 @@ void Oobe::buildServiceList()
     QSettings services_desc("/usr/share/gazelle-installer-data/services.list", QSettings::NativeFormat);
     services_desc.setIniCodec("UTF-8");
 
-    for (const QString &service : enableServices) {
+    for (const QString &service : qAsConst(enableServices)) {
         const QString &lang = QLocale::system().bcp47Name().toLower();
         QString lang_str = (lang == "en")? "" : "_" + lang;
         QStringList list = services_desc.value(service + lang_str).toStringList();
@@ -283,7 +288,7 @@ void Oobe::setService(const QString &service, bool enabled)
 {
     qDebug() << "Set service:" << service << enabled;
     if (enabled) {
-        proc.exec("update-rc.d", {service, " defaults"});
+        proc.exec("update-rc.d", {service, "defaults"});
         if (containsSystemD) proc.exec("systemctl", {"enable", service});
         if (containsRunit) {
             QFile::remove(proc.chRoot + "/etc/sv/" + service + "/down");
@@ -382,15 +387,13 @@ int Oobe::selectTimeZone(const QString &zone)
 void Oobe::setLocale()
 {
     proc.log(__PRETTY_FUNCTION__);
-    QString cmd2;
-    QString cmd;
 
     //locale
+    qDebug() << "Update locale";
+    QString cmd;
     if (!online) cmd = "chroot /mnt/antiX ";
     cmd += QString("/usr/sbin/update-locale \"LANG=%1\"").arg(gui.comboLocale->currentData().toString());
-    qDebug() << "Update locale";
     proc.shell(cmd);
-    cmd = QString("Language=%1").arg(gui.comboLocale->currentData().toString());
     const QString &selTimeZone = gui.comboTimeZone->currentData().toString();
 
     // /etc/localtime is either a file or a symlink to a file in /usr/share/zoneinfo. Use the one selected by the user.
@@ -411,8 +414,8 @@ void Oobe::setLocale()
     }
     proc.exec("hwclock", {"--hctosys"});
     if (!online) {
-        proc.exec("/bin/cp", {"-f /etc/adjtime", "/mnt/antiX/etc/"});
-        proc.exec("/bin/cp", {"-f /etc/default/rcS", "/mnt/antiX/etc/default"});
+        proc.exec("/bin/cp", {"-f", "/etc/adjtime", "/mnt/antiX/etc/"});
+        proc.exec("/bin/cp", {"-f", "/etc/default/rcS", "/mnt/antiX/etc/default"});
     }
 
     // Set clock format
@@ -428,6 +431,10 @@ void Oobe::setLocale()
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=0' /home/demo/.config/plasma-org.kde.plasma.desktop-appletsrc");
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=0' " + skelpath + "/.config/plasma-org.kde.plasma.desktop-appletsrc");
 
+        //mx fluxbox
+        proc.shell("sed -i '/time1_format/c\\time1_format=%l:%M' " + skelpath + "/.config/tint2/tint2rc");
+        proc.shell("sed -i '/time1_format/c\\time1_format=%l:%M' /home/demo/.config/tint2/tint2rc");
+
         //antix systems
         proc.shell("sed -i 's/%H:%M/%l:%M/g' " + skelpath + "/.icewm/preferences");
         proc.shell("sed -i 's/%k:%M/%l:%M/g' " + skelpath + "/.fluxbox/init");
@@ -443,6 +450,10 @@ void Oobe::setLocale()
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=2' /home/demo/.config/plasma-org.kde.plasma.desktop-appletsrc");
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=2' " + skelpath + "/.config/plasma-org.kde.plasma.desktop-appletsrc");
 
+        //mx fluxbox
+        proc.shell("sed -i '/time1_format/c\\time1_format=%H:%M' " + skelpath + "/.config/tint2/tint2rc");
+        proc.shell("sed -i '/time1_format/c\\time1_format=%H:%M' /home/demo/.config/tint2/tint2rc");
+
         //antix systems
         proc.shell("sed -i 's/%H:%M/%H:%M/g' " + skelpath + "/.icewm/preferences");
         proc.shell("sed -i 's/%k:%M/%k:%M/g' " + skelpath + "/.fluxbox/init");
@@ -639,7 +650,7 @@ void Oobe::timeAreaIndexChanged(int index)
     if (index < 0 || index >= gui.comboTimeArea->count()) return;
     const QString &area = gui.comboTimeArea->itemData(index).toString();
     gui.comboTimeZone->clear();
-    for (const QString &zone : timeZones) {
+    for (const QString &zone : qAsConst(timeZones)) {
         if (zone.startsWith(area)) {
             QString text(QString(zone).section('/', 1));
             text.replace('_', ' ');
